
/******************************************************************
 *                                                                *
 *                  Computer Science Department                   *
 *                                                                *
 *                        Master's Project                        *
 *                                                                *
 *                Graphical Development Environment               *
 *                                                                *
 *                     for Postgres Database                      *
 *                                                                *
 *                    Designed Especially for                     *
 *                                                                *
 *                 Postgres Developers and Users                  *
 *                                                                *
 *                              At                                *
 *                                                                *
 *                     Concordia University                       *
 *                                                                *
 ******************************************************************
 *                                                                *
 * Designer and Programmer:  Khaled Jababo                        *
 * Email                  :  jababo@cs.concordia.ca               *
 *                                                                *
 * Supervised by          :  Dr. Bipin C. Desai.                  *
 *                           Computer Science Department.         *
 * Email                  :  bcdesai@cs.concordia.ca              *
 *                                                                *
 * Last Modified          :  June 18, 1994                        *
 *                                                                *
 *                                                                *
 *                                                                *
 * Disclaimer:  There are no guarantees as to the accuracy, cu-   *
 *             rrency, quality, or usefulness of these programs   *
 *             or any information that is provided by using it.   *
 *             Furthermore, there is no assumption of responsi-   *
 *             bility for any los or damage which may be caused   *
 *             by their use.                                      *
 *                                                                *
 *                                                                *
 * Forward comments or questions about this report or the GDEP    *
 * system to:                                                     *
 *              bcdesai@cs.concordia.ca                           *
 *              jababo@cs.concordia.ca                            *
 *                                                                *
 ******************************************************************/


/******************************************************************/
/***********************     matools.c    *************************/
/*    This file contains all the functions needed by the main ap- */
/*  plication. For example,  initialize the  project,  update the */
/*  status file before  quitting the application,  in addition to */
/*  many Postquel functions to talk to Postgres Database.         */
/******************************************************************/

#include "mainfile.h"

/*****************************************************************/
/*                           MaFstartDb                          */
/*   Function to Set the database.                               */
/*                                                               */
/*****************************************************************/

void MaFstartDb(db)
  MaStringType db ;

  { /* begin MaFstartDb function */
    PQsetdb (db) ;
  } /* end MaFstartDb function */


/*****************************************************************/
/*                           MaFcloseDb                          */
/*    Function to Close the database.                            */
/*                                                               */
/*****************************************************************/

void MaFcloseDb()
  { /* begin MaFcloseDb function */
    PQfinish();
  } /* end   MaFcloseDb funtion*/


/*****************************************************************/
/*                        MaFdestoyClassTree                     */
/*    Function to destroy the classes in a tree hierarchy.       */
/*                                                               */
/*****************************************************************/

void MaFdestoyClassTree(MaLpStringList)
  MaStringListStruct MaLpStringList ;

  { /* begin MaFdestoyClassTree function */
    if (MaLpStringList != NULL)
      {
         MaFdestoyClassTree(MaLpStringList -> MaLpNextString) ;
         {
           MaLabelType MaLquery ;
           sprintf(MaLquery,"destroy %s", MaLpStringList -> MaLstringData) ;
           PQexec(MaLquery) ;
         }
      }
  } /* end   MaFdestoyClassTree funtion*/


/*****************************************************************/
/*                          MaFinitialize                        */
/*    Function to initialize all parameters in the application.  */
/*                                                               */
/*****************************************************************/

void MaFinitialize()
  { /* begin MaFinitialize function */
    
    MaStringType MaDB ;

    XaSystemAdministratorPrivilege = MaFALSE ;
    XaGDEPexportFileCompatible     = MaTRUE  ;

    XaMainWindowX              = 0   ;
    XaMainWindowY              = 0   ;
    XaMainWindowWidth          = 550 ;
    XaMainWindowHeight         = 800 ;
    XaWorkAreaHeaderHeight     = 95  ;
    XaWorkAreaDisplayHeight    = 300 ;
    XaWorkAreaDisplayRows      = 16  ;
    XaWorkAreaCommandHeight    = 150 ;
    XaWorkAreaCommandRows      = 4   ;
    XaWorkAreaBottomHeight     = 20  ;
    XaHelpDialogBoxTextWidth   = 300 ;
    XaHelpDialogBoxTextRows    = 6   ;
    XaHelpDialogBoxX           = 0   ;
    XaHelpDialogBoxY           = 0   ;

    XaInstanceShellBoxX        = 0   ;
    XaInstanceShellBoxY        = 0   ;
    XaInstanceAreaDisplayRows  = 2   ;

    XaShowCommandAreaStatus    = MaTRUE ;
    XaHelpWindowExistStatus    = MaFALSE ;
    XaInstanceShellExistStatus = MaFALSE ;

    strcpy(XaDisplayBuffer, "\0") ;
    strcpy(MaProjectPath,"./") ;
    strcpy(MaMosaicApplicationPath,"/pkg/Mosaic/bin/Mosaic") ;
    strcpy(MaCompressedFileExt,".Z") ;
    strcpy(MaCompressCommand,"compress") ;
    strcpy(MaUncompressCommand,"uncompress") ;

    XaWaitNoticeWindow = NULL ;

    MaKeepLog     = TRUE ;
    MaPrintHeader = TRUE ;

    sprintf(MaLogFilename,"%s%s%s",MaProjectPath,MaDbName,MaLogFilenameExt) ;
    sprintf(MaStatusFilename,"%s%s%s",MaProjectPath,MaDbName,MaStatusFilenameExt) ;

    MaFstartDb(MaDbName) ;
    strcpy(MaDB,PQdb()) ;

  } /* end   MaFinitialize funtion*/


/******************************************************************/
/*                            MaFstartUp                          */
/*    This function is to initialize the Database and all other   */
/*  parameters for the project.                                   */
/******************************************************************/

void MaFstartUp()
  { /* begin MaFstartUp function */

    printf("Initializing The Whole Project...\nPlease Wait\n") ;

    MaFinitialize() ;

    /************************************************************/
    /* Opening Log file to add the Postquel Commands that might */
    /* be executed at the end of the log file.                  */
    /************************************************************/
    if ((MaStreamLogFile = fopen(MaLogFilename,"r")) == NULL)
      {
        printf("Warning !!! Unable to open Log file : %s\n",MaLogFilename) ;
      }
    else
      {
        printf("Log File %s Has been opened.\n",MaLogFilename) ;
        strcpy(MaProjectPath, get_first_word(find_pattern_in_file
                                         ("MaProjectPath",MaLogFilename))) ;
/*
        strcpy(MaMosaicApplicationPath, get_first_word(find_pattern_in_file
                                         ("MaMosaicApplicationPath",MaLogFilename))) ;
*/
        sprintf(MaLogFilename,"%s%s%s",MaProjectPath,MaDbName,
                                       MaLogFilenameExt) ;
        sprintf(MaStatusFilename,"%s%s%s",MaProjectPath,MaDbName,
                                          MaStatusFilenameExt) ;

      }


    /************************************************************/
    /* Opening status file and reading the status of the global */
    /* variables of the project.                                */
    /************************************************************/
    if ((MaStreamStatusFile = fopen(MaStatusFilename,"r")) == NULL)
      {
        printf("Unable to open file : %s\n",MaStatusFilename) ;
      }
    else
      {
        fclose(MaStreamStatusFile) ;

        printf("Loading Status Filename %s\n",MaStatusFilename) ;

        XaSystemAdministratorPrivilege = stoi(find_pattern_in_file
                                         ("XaSystemAdministratorPrivilege",
                                          MaStatusFilename))               ;
        XaGDEPexportFileCompatible = stoi(find_pattern_in_file
                                         ("XaGDEPexportFileCompatible",
                                          MaStatusFilename))               ;
        XaMainWindowX             = stoi(find_pattern_in_file
                                         ("XaMainWindowX",MaStatusFilename))             ;
        XaMainWindowY             = stoi(find_pattern_in_file
                                         ("XaMainWindowY",MaStatusFilename))             ;
        MaKeepLog                 = stoi(find_pattern_in_file
                                         ("MaKeepLog",MaStatusFilename))                 ;
        MaPrintHeader             = stoi(find_pattern_in_file
                                         ("MaPrintHeader",MaStatusFilename))             ;
        XaMainWindowWidth         = stoi(find_pattern_in_file
                                         ("XaMainWindowWidth",MaStatusFilename))         ;
        XaMainWindowHeight        = stoi(find_pattern_in_file
                                         ("XaMainWindowHeight",MaStatusFilename))        ;
        XaWorkAreaHeaderHeight    = stoi(find_pattern_in_file
                                         ("XaWorkAreaHeaderHeight",MaStatusFilename))    ;
        XaWorkAreaDisplayHeight   = stoi(find_pattern_in_file
                                         ("XaWorkAreaDisplayHeight",MaStatusFilename))   ;
        XaWorkAreaDisplayRows     = stoi(find_pattern_in_file
                                         ("XaWorkAreaDisplayRows",MaStatusFilename))     ;
        XaWorkAreaCommandHeight   = stoi(find_pattern_in_file
                                         ("XaWorkAreaCommandHeight",MaStatusFilename))   ;
        XaWorkAreaCommandRows     = stoi(find_pattern_in_file
                                         ("XaWorkAreaCommandRows",MaStatusFilename))     ;
        XaWorkAreaBottomHeight    = stoi(find_pattern_in_file
                                         ("XaWorkAreaBottomHeight",MaStatusFilename))    ;
        XaShowCommandAreaStatus   = stoi(find_pattern_in_file
                                         ("XaShowCommandAreaStatus",MaStatusFilename))   ;
        XaHelpDialogBoxTextWidth  = stoi(find_pattern_in_file
                                         ("XaHelpDialogBoxTextWidth",MaStatusFilename))  ;
        XaHelpDialogBoxTextRows   = stoi(find_pattern_in_file
                                         ("XaHelpDialogBoxTextRows",MaStatusFilename))   ;
        XaHelpDialogBoxX          = stoi(find_pattern_in_file
                                         ("XaHelpDialogBoxX",MaStatusFilename))          ;
        XaHelpDialogBoxY          = stoi(find_pattern_in_file
                                         ("XaHelpDialogBoxY",MaStatusFilename))          ;
        XaInstanceShellBoxX       = stoi(find_pattern_in_file
                                         ("XaInstanceShellBoxX",MaStatusFilename))       ;
        XaInstanceShellBoxY       = stoi(find_pattern_in_file
                                         ("XaInstanceShellBoxY",MaStatusFilename))       ;
        XaInstanceAreaDisplayRows = stoi(find_pattern_in_file
                                         ("XaInstanceAreaDisplayRows",MaStatusFilename)) ;

        strcpy(MaDbName, get_first_word(find_pattern_in_file
                                         ("MaDbName",MaStatusFilename)))                 ;

        strcpy(MaCompressedFileExt, get_first_word(find_pattern_in_file
                                         ("MaCompressedFileExt",MaStatusFilename)))      ;

        strcpy(MaCompressCommand, get_first_word(find_pattern_in_file
                                         ("MaCompressCommand",MaStatusFilename)))        ;

        strcpy(MaUncompressCommand, get_first_word(find_pattern_in_file
                                         ("MaUncompressCommand",MaStatusFilename)))      ;

        MaFstartDb(MaDbName) ;

      }

  } /* end   MaFstartUp function */


/******************************************************************/
/*                          MaFendUp                              */
/*    This function is to close the Database, store all the para- */
/*  meters in the status filename, and finally store the Postquel */
/*  commands in the log filename.                                 */
/*                                                                */
/******************************************************************/

void MaFendUp()

  { /* begin MaFendUp function */
     MaStringType var_state ;

    XaFupdateXYposition() ;

    MaFcloseDb() ;

    /************************************************************/
    /* Closing Log file after adding the Postquel Commands that */
    /* might be added at the end of the log file.               */
    /************************************************************/
    if ((MaStreamLogFile = fopen(MaLogFilename,"w")) == NULL)
      printf("Warning Unable to Save Log File %s.\n",MaLogFilename) ;
    else
     {
       printf("Log File %s has been updated.\n",MaLogFilename) ;

       sprintf(var_state,"MaProjectPath %s\n",MaProjectPath) ;
       fprintf(MaStreamLogFile,var_state) ;
/*
       sprintf(var_state,"MaMosaicApplicationPath %s\n", MaMosaicApplicationPath) ;
       fprintf(MaStreamLogFile,var_state) ;
*/
       fclose(MaStreamLogFile) ;
     }

    /************************************************************/
    /* Opening status file and writing the status of the global */
    /* variables of the project.                                */
    /************************************************************/
         printf("Status File %s to save Current %s project status.\n"
                , MaStatusFilename,MaDbName) ;
     if ((MaStreamStatusFile = fopen(MaStatusFilename,"w")) == NULL)
       {
         printf(
            "Warning Unable to Open Status File %s to save Current %s project status.\n"
            , MaStatusFilename,MaDbName) ;
       }
     else
       {
         sprintf(var_state,"XaSystemAdministratorPrivilege %d\n",
                           XaSystemAdministratorPrivilege) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaGDEPexportFileCompatible %d\n",
                           XaGDEPexportFileCompatible) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaKeepLog %d\n",MaKeepLog) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaPrintHeader %d\n",MaPrintHeader) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaDbName %s\n",MaDbName) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaMainWindowX %d\n",XaMainWindowX) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaMainWindowY %d\n",XaMainWindowY) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaMainWindowWidth %d\n",XaMainWindowWidth) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaMainWindowHeight %d\n",XaMainWindowHeight) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaHeaderHeight %d\n",XaWorkAreaHeaderHeight) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaDisplayHeight %d\n",XaWorkAreaDisplayHeight) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaDisplayRows %d\n",XaWorkAreaDisplayRows) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaCommandHeight %d\n",XaWorkAreaCommandHeight) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaCommandRows %d\n",XaWorkAreaCommandRows) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaWorkAreaBottomHeight %d\n",XaWorkAreaBottomHeight) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaShowCommandAreaStatus %d\n",XaShowCommandAreaStatus) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaHelpDialogBoxTextWidth %d\n",XaHelpDialogBoxTextWidth) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaHelpDialogBoxTextRows %d\n",XaHelpDialogBoxTextRows) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaHelpDialogBoxX %d\n",XaHelpDialogBoxX) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaHelpDialogBoxY %d\n",XaHelpDialogBoxY) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaInstanceShellBoxX %d\n",XaInstanceShellBoxX) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaInstanceShellBoxY %d\n",XaInstanceShellBoxY) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"XaInstanceAreaDisplayRows %d\n",XaInstanceAreaDisplayRows) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaCompressedFileExt %s\n",MaCompressedFileExt) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaCompressCommand %s\n",MaCompressCommand) ;
         fprintf(MaStreamStatusFile,var_state) ;

         sprintf(var_state,"MaUncompressCommand %s\n",MaUncompressCommand) ;
         fprintf(MaStreamStatusFile,var_state) ;

         fclose(MaStreamStatusFile) ;
       }

  } /* end   MaFendUp function */


/*****************************************************************/
/*                     MaFgetPqCommandResult                     */
/*    Function to return the contents of a portal buffer.        */
/*                                                               */
/*****************************************************************/

char *MaFgetPqCommandResult(MaLportalName)
  char *MaLportalName ;

  { /* begin MaFgetPqCommandResult function */
    int          MaLi, MaLj, MaLk, MaLnGroups, MaLnTuplesGroup,
                 MaLnFieldsGroup , MaLt;
    char         MaLtempString[30] ;
    MaStringType MaLstring  ;
    PortalBuffer *MaLportal ;
    MaBufferType MaLresultBuffer ;

    strcpy(MaLresultBuffer,"Returning the following Database Result\n") ;

    sprintf(MaLstring,"fetch all in %s",MaLportalName) ;
    PQexec(MaLstring) ;

    MaLportal       = PQparray(MaLportalName) ;
    MaLnGroups      = PQngroups(MaLportal);
    MaLt            = 0 ;

    for(MaLk=0; MaLk < MaLnGroups; MaLk++)
      {
        if (MaLk>0)
          strcat(MaLstring,"\nA new instance group : \n");
        /* number of objects for the class */
        MaLnTuplesGroup = PQntuplesGroup(MaLportal, MaLk);
        /* number of attributes for the class */
        MaLnFieldsGroup  = PQnfieldsGroup(MaLportal, MaLk);
      
        /* find out what the name of the attributes are */
        for(MaLi=0; MaLi < MaLnFieldsGroup ; MaLi++)
          {
            sprintf(MaLstring,"%s", PQfnameGroup(MaLportal, MaLk, MaLi)) ;
            if (strlen(MaLstring) > 17)
              {
                MaLstring[17] = '*'  ;
                MaLstring[18] = '\0' ;
              }
            sprintf(MaLtempString, "%-20s", MaLstring) ;
            strcat(MaLresultBuffer,MaLtempString);
          }
        strcpy(MaLstring,"\n");
        strcat(MaLresultBuffer,MaLstring);
      
        /* print the value of each attribute for each object */
        for(MaLi=0; MaLi<MaLnTuplesGroup; MaLi++)
          {
            for(MaLj=0; MaLj < MaLnFieldsGroup ; MaLj++)
              {
                /* read the ith attribute of the ith object */
                sprintf(MaLstring,"%s", PQgetvalue(MaLportal, MaLt+MaLi, MaLj));
                if (strlen(MaLstring) > 17)
                  {
                    MaLstring[17] = '*'  ;
                    MaLstring[18] = '\0' ;
                  }
                sprintf(MaLtempString, "%-20s", MaLstring) ;
                if ((strlen(MaLresultBuffer)+strlen(MaLtempString)) < (MaMaxBufSize-5))
                  strcat(MaLresultBuffer,MaLtempString) ;
              }
            strcpy(MaLstring,"\n");
            strcat(MaLresultBuffer,MaLstring);
          }
        MaLt += MaLnTuplesGroup ;
      }

    sprintf(MaLstring,"close %s",MaLportalName) ;
    PQexec(MaLstring) ;
    
    return(MaLresultBuffer) ;

  } /* end   MaFgetPqCommandResult function */


/*****************************************************************/
/*                     MaFgetAvailablePortalName                 */
/*    Function to return the name of an available portal.        */
/*                                                               */
/*****************************************************************/

char *MaFgetAvailablePortalName()

  { /* begin MaFgetAvailablePortalName function */
    int MaLindex = 0 ;
    MaLabelType MaLportalName ;

    while ((MaLindex < MaMaxPortalNameNum) && 
            (MaGportalArrayAvailable[MaLindex] == MaFALSE))
      {
        MaLindex++ ;
      }

    if (MaGportalArrayAvailable[MaLindex] == MaFALSE)
      {
        printf("Warning Exceeded Portal Number Available\n") ;
        return("eportal") ;
      }

    MaGportalArrayAvailable[MaLindex] = MaFALSE ;
    sprintf(MaLportalName,"eportal%d",MaLindex) ;
    return(MaLportalName) ;

  } /* end   MaFgetAvailablePortalName function */


/*****************************************************************/
/*                       MaFreleasePortalName                    */
/*    Function to release the portal to be able to reuse it by   */
/*  other queries.                                               */
/*                                                               */
/*****************************************************************/

void MaFreleasePortalName(MaLportalName)
  char *MaLportalName ;

  { /* begin MaFreleasePortalName function */
    int MaLvalue = stoi(&(MaLportalName[7])) ;

    MaGportalArrayAvailable[MaLvalue] = MaTRUE ;

  } /* end   MaFreleasePortalName function */


/*****************************************************************/
/*                        MaFexePqCommand                        */
/*    Function to issue a PostQuel command to the Database.      */
/*                                                               */
/*****************************************************************/

char *MaFexePqCommand(XaLcommand)
  char *XaLcommand ;

  { /* begin MaFexePqCommand function */
    char         *XaLret ;
    char         *XaLstring ;
    char         *XaLportalName = MaFgetAvailablePortalName() ;
    char         *XaLpattern = "retrieve" ;

    MaBufferType XaLbuffer ;
    MaBufferType XaLretrieveQuery ;
    MaBufferType XaLqueryCommand ;
    MaBufferType MaLreturnBuffer ;

    if (XaLcommand[0] == '\0')
      {
        XtFree(XaLretrieveQuery) ;
        XtFree(XaLqueryCommand) ;
        XtFree(XaLbuffer) ;

        strcpy(MaLreturnBuffer, "Sorry, Could Not Execute an Empty String\n" ) ;
        strcat(MaLreturnBuffer, "Please Enter a Command in the Command Area\n") ;
        MaFreleasePortalName(XaLportalName) ;
        return(MaLreturnBuffer) ;
      }
    strcpy(XaLretrieveQuery, MaFpatternInString (XaLpattern,XaLcommand)) ;

    if (XaLretrieveQuery[0] != '\0')
      {
        sprintf(XaLqueryCommand,"retrieve portal %s %s",
                                XaLportalName, XaLretrieveQuery) ;
      }	
    else
      strcpy(XaLqueryCommand,XaLcommand) ;

    sprintf(XaLbuffer, "Query %s sent to backend process.\n",XaLqueryCommand) ;
    PQexec ("begin");

    XaLret = PQexec (XaLqueryCommand);
    if (*XaLret == 'R')
      {
        strcat(XaLbuffer,"Query Could not be executed.\n") ;
      }
    else if (strncmp("CRETRIEVE",XaLret,9) == 0)
      {
        MaBufferType XaLtempBuffer ;
        strcat(XaLbuffer, "Result Of Query :\n") ;
        strcpy(XaLtempBuffer, MaFgetPqCommandResult(XaLportalName)) ;

        if ((strlen(XaLbuffer) + strlen(XaLtempBuffer)) < (MaMaxBufSize - 5))
          strcat(XaLbuffer, XaLtempBuffer) ;
        else
          {
            (void) XaFisEmptyString(XaWorkAreaDisplay,"",
                                    "Warning Result Query Excceeds Buffer size Limit") ;
            XaLtempBuffer[MaMaxBufSize - strlen(XaLbuffer) - 5] = '\0' ;
            strcat(XaLbuffer, XaLtempBuffer) ;
          }
      }
    else
      strcat(XaLbuffer, "Query Has Been Executed Successfully.\n") ;

    PQexec("end");
    strcpy(MaLreturnBuffer,XaLbuffer) ;

    XtFree(XaLret) ;
    XtFree(XaLstring) ;
    XtFree(XaLretrieveQuery) ;
    XtFree(XaLportalName) ;
    XtFree(XaLqueryCommand) ;
    XtFree(XaLbuffer) ;
    MaFreleasePortalName(XaLportalName) ;
    return(MaLreturnBuffer) ;

  } /* end   MaFexePqCommand function */


/*****************************************************************/
/*                 MaFgetStringListExePqCommandResult            */
/*    Function to return a pointer to a link list that contains  */
/*  the contents of a particular portal name buffer.             */
/*                                                               */
/*****************************************************************/

MaStringListStruct MaFgetStringListExePqCommandResult(MaLportalName)
  char *MaLportalName ;

  { /* begin MaFgetStringListExePqCommandResult function */
    int          MaLi, MaLj, MaLk, MaLnGroups, MaLnTuplesGroup,
                 MaLnFieldsGroup , MaLt;
    char         MaLstring[200] ;
    char         MaLtempString[200] ;
    PortalBuffer *MaLportal ;
    MaStringListStruct MaLresultPointer = NULL ;
    extern MaStringListStruct MaFpointerStringToListAdd() ;

    sprintf(MaLstring,"fetch all in %s",MaLportalName) ;
    PQexec(MaLstring) ;

    MaLportal       = PQparray(MaLportalName) ;
    MaLnGroups      = PQngroups(MaLportal);
    MaLt            = 0 ;

    for(MaLk=0; MaLk < MaLnGroups; MaLk++)
      {
        if (MaLk>0)
          strcat(MaLstring,"\nA new instance group : \n");
        /* number of objects for the class */
        MaLnTuplesGroup = PQntuplesGroup(MaLportal, MaLk);
        /* number of attributes for the class */
        MaLnFieldsGroup  = PQnfieldsGroup(MaLportal, MaLk);
      
        /***************************************************/
        /* Add the value of each attribute for each object */
        /* to the list.                                    */
        /***************************************************/
        for(MaLi=0; MaLi<MaLnTuplesGroup; MaLi++)
          {
            strcpy(MaLstring,"\0");
            for(MaLj=0; MaLj < MaLnFieldsGroup ; MaLj++)
              {
                if (MaLj > 0)
                  strcat(MaLstring, " ") ;
                /* read the ith attribute of the ith object */
                sprintf(MaLtempString,"%s", PQgetvalue(MaLportal, MaLt+MaLi, MaLj)) ;
                strcat(MaLstring, MaLtempString);
              }
            MaLresultPointer = MaFpointerStringToListAdd(MaLstring, MaLresultPointer) ;
          }
        MaLt += MaLnTuplesGroup ;
      }

    sprintf(MaLstring,"close %s",MaLportalName) ;
    PQexec(MaLstring) ;

    return(MaLresultPointer) ;

  } /* end   MaFgetStringListExePqCommandResult function */


/*****************************************************************/
/*                   MaFgetStringListExePqCommand                */
/*    Function to issue a PostQuel command to the Database and   */
/*  returns the result in a MaStringListStruct structue list.    */
/*                                                               */
/*****************************************************************/

MaStringListStruct MaFgetStringListExePqCommand(XaLcommand)
  char *XaLcommand ;

  { /* begin MaFgetStringListExePqCommand function */
    char         *XaLret ;
    char         *XaLportalName = MaFgetAvailablePortalName() ;
    char         *XaLpattern = "retrieve"   ;

    MaBufferType XaLretrieveQuery ;
    MaBufferType XaLqueryCommand  ;

    if (XaLcommand[0] == '\0')
      {
        MaFreleasePortalName(XaLportalName) ;
        return(NULL) ;
      }
    strcpy(XaLretrieveQuery, MaFpatternInString (XaLpattern,XaLcommand)) ;

    if (XaLretrieveQuery[0] != '\0')
      {
        sprintf(XaLqueryCommand,"retrieve portal %s %s",
                                XaLportalName, XaLretrieveQuery) ;
      }	
    else
      strcpy(XaLqueryCommand,XaLcommand) ;

    PQexec ("begin");

    XaLret = PQexec (XaLqueryCommand);
    if (*XaLret == 'R')
      {
        PQexec("end");
        MaFreleasePortalName(XaLportalName) ;
        return(NULL) ;
      }
    if (strncmp("CRETRIEVE",XaLret,9) == 0)
      {
        MaStringListStruct 
           MaStringListResult = MaFgetStringListExePqCommandResult(XaLportalName) ;
        PQexec("end") ;
        MaFreleasePortalName(XaLportalName) ;
        return(MaStringListResult) ;
      }
    PQexec("end");
    MaFreleasePortalName(XaLportalName) ;
    return(NULL) ;

  } /* end   MaFgetStringListExePqCommand function */


/*****************************************************************/
/*                       MaFgetAllCildrenList                    */
/*    Function to return a pointer containing a list of all the  */
/*  subclasses of the root class.                                */
/*                                                               */
/*****************************************************************/

MaStringListStruct MaFgetAllCildrenList(MaLrootClass)
  char *MaLrootClass ;

  { /* begin MaFgetAllCildrenList function */
    MaStringListStruct MaLchildrenList ;
    MaStringListStruct MaLtempChildNode ;
    MaStringListStruct MaLintermediateChildrenList ;
    MaStringListStruct MaLcurrentRootClassNode ;
    MaStringType       MaLquery ;

    extern int MaFwordInStringList () ;
    extern MaStringListStruct MaFfreeStringList () ;
    extern MaStringListStruct MaFpointerNodeToListAdd() ;

    sprintf(MaLquery,"retrieve (child=hijo.relname) \
                      from  clase, hijo in pg_class, hereda in pg_inherits \
                      where hereda.inhparent = clase.oid \
                            and hereda.inhrel = hijo.oid \
                            and clase.relname =  \"%s\"", MaLrootClass) ;

    MaLchildrenList = MaFgetStringListExePqCommand(MaLquery) ;

    MaLcurrentRootClassNode = MaLchildrenList ;
    while(MaLcurrentRootClassNode != NULL)
      {
        sprintf(MaLquery,"retrieve (child=hijo.relname) \
                      from  clase, hijo in pg_class, hereda in pg_inherits \
                      where hereda.inhparent = clase.oid \
                            and hereda.inhrel = hijo.oid \
                            and clase.relname =  \"%s\"",
                MaLcurrentRootClassNode -> MaLstringData) ;

        MaLintermediateChildrenList = MaFgetStringListExePqCommand(MaLquery) ;

        while (MaLintermediateChildrenList != NULL)
          {
            MaLtempChildNode = MaLintermediateChildrenList ;
            MaLintermediateChildrenList = MaLintermediateChildrenList->MaLpNextString ;
            MaLtempChildNode -> MaLpNextString = NULL ;

            if (MaFwordInStringList (MaLtempChildNode -> MaLstringData,
                                     MaLchildrenList))
              {
                MaLtempChildNode = MaFfreeStringList(MaLtempChildNode) ;
              }
            else
              {
                MaLchildrenList = MaFpointerNodeToListAdd(MaLtempChildNode,
                                                          MaLchildrenList) ;
              }

          }

        MaLcurrentRootClassNode = MaLcurrentRootClassNode -> MaLpNextString ;
      }

    return(MaLchildrenList) ;

  } /* end   MaFgetAllCildrenList function */


/*****************************************************************/
/*                   MaFgetQueryAttributesNameValue              */
/*    Function to take an argument a Client Data Pointer, look   */
/*  for the current Oid, and set the instance variables of the   */
/*  current Oid to their values.                                 */
/*                                                               */
/*****************************************************************/

MaStringListStruct MaFgetQueryAttributesNameValue(MaLportalName)
  char *MaLportalName ;

  { /* begin MaFgetQueryAttributesNameValue function */

    int          MaLi, MaLj, MaLk, MaLnGroups, MaLnTuplesGroup,
                 MaLnFieldsGroup , MaLt;
    char         MaLstring[2000] ;
    char         MaLtempString[2000] ;
    PortalBuffer *MaLportal ;
    MaStringListStruct MaLresultPointer = NULL ;
    MaStringListStruct MaLpTempData ;
    extern MaStringListStruct MaFpointerStringToListAdd() ;
    MaStringListStruct extern MaFgetPtrStringList() ;
    MaStringListStruct extern MaFpointerNodeToListAdd() ;

    for (MaLi = 0; MaLi < 200; MaLi++)
      {
        MaLstring[MaLi] = ' ';
        MaLtempString[MaLi] = ' ' ;
      }
    MaLstring[0] = '\0';
    MaLtempString[0] = '\0' ;

    sprintf(MaLstring,"fetch all in %s",MaLportalName) ;
    PQexec(MaLstring) ;

    MaLportal       = PQparray(MaLportalName) ;
    MaLnGroups      = PQngroups(MaLportal);
    MaLt            = 0 ;

    for(MaLk=0; MaLk < MaLnGroups; MaLk++)
      {
        if (MaLk>0)
          strcat(MaLstring,"\nA new instance group : \n");
        /* number of objects for the class */
        MaLnTuplesGroup = PQntuplesGroup(MaLportal, MaLk);
        /* number of attributes for the class */
        MaLnFieldsGroup  = PQnfieldsGroup(MaLportal, MaLk);
      
        /***************************************************/
        /* Add the value of each attribute for each object */
        /* to the list.                                    */
        /***************************************************/
        for(MaLi=0; MaLi<MaLnTuplesGroup; MaLi++)
          {
            for(MaLj = 0; MaLj < MaLnFieldsGroup ; MaLj++)
              {
                strcpy(MaLstring,"\0");
                /* read the ith attribute of the ith object */
                if (PQgetvalue(MaLportal, MaLt+MaLi, MaLj) != NULL)
                  strcpy(MaLstring, PQgetvalue(MaLportal, MaLt+MaLi, MaLj)) ;
                MaLpTempData = MaFgetPtrStringList(MaLstring) ;

                strcpy(MaLstring, PQfnameGroup(MaLportal, MaLk, MaLj)) ;
                strcpy(MaLpTempData -> MaLlabelName, MaLstring) ;
                MaLresultPointer = MaFpointerNodeToListAdd
                                       (MaLpTempData, MaLresultPointer) ;
              }
          }
        MaLt += MaLnTuplesGroup ;
      }

    sprintf(MaLstring,"close %s", MaLportalName) ;
    PQexec(MaLstring) ;
    return(MaLresultPointer) ;

  } /* end   MaFgetQueryAttributesNameValue function */


/*****************************************************************/
/*                MaFsetOidsInstanceAttributesValue              */
/*    Function to take an argument a Client Data Pointer, look   */
/*  for the current Oid, and set the instance variables of the   */
/*  current Oid to their values.                                 */
/*                                                               */
/*****************************************************************/

void MaFsetOidsInstanceAttributesValue(XaLpClientData)
  MaClientDataStruct XaLpClientData ;

  { /* begin MaFsetOidsInstanceAttributesValue function */
    char         *XaLret ;
    int           MaLfoundOid = MaFALSE ;
    MaStringType  MaLquery ;
    MaLabelType   XaLportalName ; 
    MaStringListStruct XaLpValuesOidList = NULL ;
    MaStringListStruct XaLpCurrentOidList = XaLpClientData -> MaLobjectIdsList ;

    strcpy(XaLportalName, MaFgetAvailablePortalName()) ;
    if (XaLpCurrentOidList == NULL || XaLpClientData -> MaLcurrentOid == 0)
      return ;
    
    sprintf(MaLquery,"retrieve portal %s (e.all) from e in %s where\
                      e.oid = \"%d\" ", XaLportalName,
                      XaLpClientData -> MaLclassName,
                      XaLpClientData -> MaLcurrentOid) ;

    PQexec ("begin");
    XaLret = PQexec (MaLquery);
    if (*XaLret == 'R')
      {
        PQexec("end");
        MaFreleasePortalName(XaLportalName) ;
        return ;
      }
    if (strncmp("CRETRIEVE",XaLret,9) == 0)
      {
        XaLpValuesOidList = MaFgetQueryAttributesNameValue(XaLportalName) ;
      }
    else
      {
        PQexec("end");
        MaFreleasePortalName(XaLportalName) ;
        return ;
      }

    PQexec("end") ;
    MaFreleasePortalName(XaLportalName) ;
    /********************************************************************/
    /*   Set Values with regard to XaLpValuesOidList pointer contents   */
    /********************************************************************/
    while (XaLpValuesOidList != NULL)
      {
        MaStringListStruct XaLpCurrentField = XaLpValuesOidList ;
        MaStringListStruct XaLpSearchClientData = XaLpClientData -> MaLstringList ;
        int XaLfound = MaFALSE ;

        while ((XaLpSearchClientData != NULL) && (XaLfound == MaFALSE))
          {
            if (strcmp(XaLpSearchClientData -> MaLlabelName,
                       XaLpValuesOidList    -> MaLlabelName) == 0)
              XaLfound = MaTRUE ;
            else
              XaLpSearchClientData = XaLpSearchClientData -> MaLpNextString ;
          }

        if (XaLfound == MaFALSE)
          {
            XaLpSearchClientData = XaLpClientData -> MaLtextStringList ;
            while ((XaLpSearchClientData != NULL) && (XaLfound == MaFALSE))
              {
                if (strcmp(XaLpSearchClientData -> MaLlabelName,
                           XaLpValuesOidList    -> MaLlabelName) == 0)
                  XaLfound = MaTRUE ;
                else
                  XaLpSearchClientData = XaLpSearchClientData -> MaLpNextString ;
              }
          }

        if  (XaLfound)
          {
            strcpy(XaLpSearchClientData -> MaLstringData,
                   XaLpValuesOidList    -> MaLstringData) ;
          }

        XaLpValuesOidList = XaLpValuesOidList -> MaLpNextString ;
        XaLpCurrentField -> MaLpNextString = NULL ;
        free(XaLpCurrentField) ;
      }

  } /* end   MaFsetOidsInstanceAttributesValue function */


/*****************************************************************/
/*                 MaFcheckConflictParentChildType               */
/*    Function to check if there is a conflct between the Parent */
/*  class and the children class then popup a dialog box stating */
/*  the nature of conflict and returns false to the calling pro- */
/*  gram, otherwise it returns true.                             */
/*                                                               */
/*****************************************************************/

int MaFcheckConflictParentChildType(XaLwidget, XaLpClientData)
  Widget             XaLwidget      ;
  MaClientDataStruct XaLpClientData ;

  { /* begin MaFcheckConflictParentChildType function */
    extern MaClientDataStruct MaFgetPtrClientData() ;
    extern MaClientDataStruct MaFfreeClientData () ;
    extern MaStringListStruct MaFpointerNodeToListAdd() ;

    MaClientDataStruct XaLpNewParentClientData ;
    MaClientDataStruct XaLpChildClientData ;

    MaStringListStruct XaLpCurrentChildNode = XaLpClientData -> MaLchildrenStringList ;
    MaStringListStruct XaLpParentStringList   ;
    MaStringListStruct XaLpChildrenStringList ;

    int XaLfoundAttributeName ;

    XaLpNewParentClientData = MaFgetPtrClientData() ;
    strcpy(XaLpNewParentClientData -> MaLclassName,
           XaLpClientData -> MaLnewParentName) ;

    /*************************************************************/
    /*  Retrieve the New Parent Class Attributes and their types */
    /*************************************************************/
    if (!XaFsetClassAttributeNameType(XaLwidget, XaLpNewParentClientData))
      {
        XaLpNewParentClientData = MaFfreeClientData (XaLpNewParentClientData) ;

        sprintf(XaLpClientData -> MaLdialogMessage, 
               "Could Not Create Instance Variable Types for Parent Class %s",
                XaLpClientData -> MaLnewParentName) ;

        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;

        XaFsetRegularCursorFont() ;

        XaFpostDialog(XtParent(XaLwidget) ,XaLpClientData) ;
        return(MaFALSE) ;
      }

    XaLpNewParentClientData -> MaLstringList =
               MaFpointerNodeToListAdd(XaLpNewParentClientData -> MaLtextStringList,
                                       XaLpNewParentClientData -> MaLstringList) ;
    XaLpNewParentClientData -> MaLtextStringList = NULL ;

    /*************************************************************/
    /*  Check if there is conflict between Parent Class and any  */
    /*  of the class tree.  If  same instance variable name but  */
    /*  have different type.                                     */
    /*************************************************************/
    while ((XaLpClientData -> MaLparentChildConflict == MaFALSE) && 
           (XaLpCurrentChildNode != NULL))
      {
        XaLpChildClientData = MaFgetPtrClientData() ;
        strcpy(XaLpChildClientData -> MaLclassName,
               XaLpCurrentChildNode -> MaLstringData) ;

        /**********************************************************/
        /*  Retrieve the Tree SubClass Attributes and their types */
        /**********************************************************/
        if (!XaFsetClassAttributeNameType(XaLwidget, XaLpChildClientData))
          {
            XaLpChildClientData = MaFfreeClientData (XaLpChildClientData) ;

            sprintf(XaLpClientData -> MaLdialogMessage, 
               "Could Not Create Instance Variable Types for Class %s in the Tree",
                XaLpCurrentChildNode -> MaLstringData) ;

            XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;

            XaFsetRegularCursorFont() ;

            XaFpostDialog(XtParent(XaLwidget) ,XaLpClientData) ;
            return(MaFALSE) ;
          } /* end if Block */

        XaLpChildClientData -> MaLstringList =
               MaFpointerNodeToListAdd(XaLpChildClientData -> MaLtextStringList,
                                       XaLpChildClientData -> MaLstringList) ;
        XaLpChildClientData -> MaLtextStringList = NULL ;

        /********************************************************************/
        /*  Check if there is conflict in types between parent and SubClass */
        /********************************************************************/
        XaLpParentStringList   = XaLpNewParentClientData -> MaLstringList ;

        while (XaLpParentStringList != NULL)
          {
            XaLfoundAttributeName = MaFALSE ;
            XaLpChildrenStringList = XaLpChildClientData -> MaLstringList ;

            while ((XaLpChildrenStringList != NULL) &&
                   (XaLfoundAttributeName == MaFALSE))
              {
                /* Check For Conflicts of attributes types */
                if (strcmp(XaLpParentStringList   -> MaLlabelName,
                           XaLpChildrenStringList -> MaLlabelName) == 0)
                  {
                    XaLfoundAttributeName = MaTRUE ;
                    if (strcmp(XaLpParentStringList   -> MaLlabelType,
                               XaLpChildrenStringList -> MaLlabelType) != 0)
                      {

                        sprintf(XaLpClientData -> MaLdialogMessage, 
                                "Could Not Change Parent for Class %s due to Conflict\
 between Parent %s and Class %s on Field %s Type",
                                XaLpClientData -> MaLclassName,
                                XaLpNewParentClientData -> MaLclassName,
                                XaLpChildClientData     -> MaLclassName,
                                XaLpParentStringList    -> MaLlabelName) ;

                        XaLpNewParentClientData =
                                           MaFfreeClientData (XaLpNewParentClientData) ;
                        XaLpChildClientData  = MaFfreeClientData (XaLpChildClientData) ;

                        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;

                        XaFsetRegularCursorFont() ;

                        XaFpostDialog(XtParent(XaLwidget) ,XaLpClientData) ;

                        return(MaFALSE) ;
                      }

                  }

                XaLpChildrenStringList = XaLpChildrenStringList -> MaLpNextString ;
              }

            XaLpParentStringList = XaLpParentStringList -> MaLpNextString ;
          }

        XaLpChildClientData  = MaFfreeClientData (XaLpChildClientData) ;
        XaLpCurrentChildNode = XaLpCurrentChildNode -> MaLpNextString ;

      } /* end while Block */

    XaLpNewParentClientData = MaFfreeClientData (XaLpNewParentClientData) ;

    return(MaTRUE) ;

  } /* end   MaFcheckConflictParentChildType function */


/*****************************************************************/
/*                                                               */
/*                        END OF matools.c FILE                  */
/*                                                               */
/*****************************************************************/